@@ -23,6 +23,92 @@ def validate_data(data: dict):
             >>> validate_data(bad_gdf_2)
             Exception: "emotives" must be a dict of <str, float>. Dict not provided!
 
+    .. testsetup:: validate_data
+
+        gdf1 = {"strings": [],
+            "vectors": [],
+            "emotives": {},
+            "metadata": {}}
+
+        # empty gdf with additional field
+        gdf2 = {"strings": [],
+                "vectors": [],
+                "emotives": {},
+                "metadata": {},
+                "invalid": True}
+
+        # gdf with single string
+        gdf3 = {"strings": ["hello"],
+                "vectors": [],
+                "emotives": {},
+                "metadata": {}
+                }
+
+        # gdf with single string and vector
+        gdf4 = {"strings": ["hello"],
+                "vectors": [[1, 2, 3, 4]],
+                "emotives": {},
+                "metadata": {}
+                }
+
+        # gdf with malformed strings field
+        gdf5 = {"strings": "hello",
+                "vectors": [],
+                "emotives": {},
+                "metadata": {}
+                }
+
+        # gdf with malformed vectors field
+        gdf6 = {"strings": ["hello"],
+                "vectors": [1, 2, 3, 4],
+                "emotives": {},
+                "metadata": {}
+                }
+
+        # gdf with malformed emotives field
+        gdf7 = {"strings": ["hello"],
+                "vectors": [[1, 2, 3, 4]],
+                "emotives": {"utility": "high"},
+                "metadata": {}
+                }
+
+        # gdf with valid emotives field
+        gdf8 = {"strings": ["hello"],
+                "vectors": [[1, 2, 3, 4]],
+                "emotives": {"utility": 23.7},
+                "metadata": {}
+                }
+                
+        from ia.gaius.data_ops import validate_data
+
+    .. doctest:: validate_data
+        :hide:
+        
+        >>> validate_data(gdf1)
+        True
+        >>> validate_data(gdf2)
+        Traceback (most recent call last):
+        ...
+        Exception: Key: %s, should not be in the data dictionary!
+        >>> validate_data(gdf3)
+        True
+        >>> validate_data(gdf4)
+        True
+        >>> validate_data(gdf5)
+        Traceback (most recent call last):
+        ...
+        Exception: "strings" must be a list of strings.  List not provided!
+        >>> validate_data(gdf6)
+        Traceback (most recent call last):
+        ...
+        Exception: "vectors" must be a list of arrays (i.e. lists)!
+        >>> validate_data(gdf7)
+        Traceback (most recent call last):
+        ...
+        Exception: "emotives" must be a dict of <str, float>!
+        >>> validate_data(gdf8)
+        True
+        
     """
     if not isinstance(data, dict):
         raise Exception("Incorrect data type.  Must be a dictionary.")
@@ -41,7 +127,7 @@ def validate_data(data: dict):
 
     if not isinstance(data["emotives"], dict):
         raise Exception('"emotives" must be a dict of <str, float>. Dict not provided!')
-    for item in data["emotives"]:
+    for item in data["emotives"].values():
         if not isinstance(item, (int, float)):
             raise Exception('"emotives" must be a dict of <str, float>!')
 
