@@ -278,10 +278,10 @@ def recursive_delete_model(agent: AgentClient, model_name: str, nodes=None, log_
     """Recursively remove a model from an Agent.
 
     Args:
-        agent (AgentClient): GAIuS Agent
-        model_name (str): name of model to delete
-        nodes (_type_, optional): Nodes to recursively delete model on
-        log_to_file (bool, optional): Whether to log output to a file. Defaults to False.
+        agent (AgentClient): _description_
+        model_name (str): _description_
+        nodes (_type_, optional): _description_. Defaults to None.
+        log_to_file (bool, optional): _description_. Defaults to False.
     """
 
     if not agent._connected:
@@ -289,37 +289,41 @@ def recursive_delete_model(agent: AgentClient, model_name: str, nodes=None, log_
 
     if log_to_file:
         import time
-        file = open(f"{time.now()}_log.txt")
-
+        t = time.localtime()
+        file = open(f"{time.strftime('recursive_model_delete_log_%Y_%m_%d_%H_%M_%S', t)}_log.txt", mode='w+')
+    
     # variables
-    # changed_models = {}
+    changed_models = {}
     primitve_patterns_to_remove = []
     symbols_to_update_in_other_nodes = {}
-    # node_names = [node["name"] for node in agent.all_nodes]
-    # node_ids = [node["id"] for node in agent.all_nodes]
-
+    
     if nodes is None:
         nodes = agent.all_nodes
     else:
         nodes = [node for node in agent.all_nodes if (node['name'] in nodes)]
 
-    result = {}
-
-    print(f"nodes = {nodes}")
-
+    overall_result = {}
+    
+    for node in nodes:
+        changed_models[node["name"]] = dict()
+    
     for node in nodes:
         try:
             # call delete model on the passed nodes
             response = agent.delete_model(model_name, [node["name"]])
-
-            print(f"initial delete_model response = {response}")
-
+            overall_result[node["name"]] = response[node["name"]]
+            
+            if(log_to_file):
+                file.write(f"initial delete_model response = {response}")
+            
             # if model not in node, need to look for patterns
-            # else add the pattern with the passed model name to primitve_patterns_to_remove
-            if response[node["name"]] != "deleted":
-                print("Trying to recursive delete models")
+            # else add the pattern with the passed model name to primitve_patterns_to_remove 
+            if(response[node["name"]] != "deleted"):
+                if(log_to_file):
+                    file.write("Trying to recursive delete models")
                 for pattern in primitve_patterns_to_remove:
-                    print(f"Looking for pattern {pattern} in {node['name']}")
+                    if(log_to_file):
+                        file.write(f"Looking for pattern {pattern} in {node['name']}")
                     result = agent.remove_patterns_from_system([pattern])
 
                     # go through nodes again to make sure that pattern gets removed
@@ -329,20 +333,24 @@ def recursive_delete_model(agent: AgentClient, model_name: str, nodes=None, log_
                         continue
 
                     for node1 in nodes:
-                        print(f"node1 = {node1['name']}")
+                        if(log_to_file):
+                            file.write(f"node1 = {node1['name']}")
                         # if nothing was found, then go to next pattern
-                        print(f"result[{node1['name']}] = {result[node1['name']] if (result[node1['name']] == {}) else result[node1['name']]['models']}")
-
-                        if result[node1["name"]] == {}:
+                        if(log_to_file):
+                            file.write(f"result[{node1['name']}] = {result[node1['name']] if (result[node1['name']] == {}) else result[node1['name']]['models']}")
+                        
+                        if(result[node1["name"]] == {}):
                             continue
 
                         for old_model_hash, new_model_hash in result[node1["name"]]["models"].items():
                             # so if that caused a model to be deleted, add this model
                             # to list of patterns to remove
 
-                            # else update models further
-                            if new_model_hash == "deleted":
-                                print(f"Adding .+\|{node1['id']}\|{old_model_hash}.+ to primitve_patterns_to_remove")
+                            # else update models further 
+                            if(new_model_hash == "deleted"):
+                                if(log_to_file):
+                                    file.write(f"Adding .+\|{node1['id']}\|{old_model_hash}.+ to primitve_patterns_to_remove")
+                                
                                 pattern = f".+\|{node1['id']}\|{old_model_hash}.+"
 
                                 # don't add the pattern if its already there
@@ -350,16 +358,21 @@ def recursive_delete_model(agent: AgentClient, model_name: str, nodes=None, log_
                                     continue
 
                                 primitve_patterns_to_remove.append(f".+\|{node1['id']}\|{old_model_hash}.+")
+                                changed_models[node1["name"]][old_model_hash] = "deleted"
                             else:
-                                print(f"Adding .+\|{node1['id']}\|{old_model_hash}.+ to symbols_to_update_in_other_nodes")
+                                if(log_to_file):
+                                    file.write(f"Adding .+\|{node1['id']}\|{old_model_hash}.+ to symbols_to_update_in_other_nodes")
                                 pattern = f".+{old_model_hash}.+"
                                 # don't add the pattern if its already there
                                 if pattern in symbols_to_update_in_other_nodes:
                                     continue
 
                                 symbols_to_update_in_other_nodes[f".+{old_model_hash}.+"] = new_model_hash
-
-                print("Updating symbols")
+                                changed_models[node1["name"]][old_model_hash] = new_model_hash
+                    
+                if(log_to_file):
+                    file.write(f"Updating symbols")
+                
                 for old_symbol, new_symbol in symbols_to_update_in_other_nodes.items():
                     '''
                     Steps:
@@ -374,12 +387,14 @@ def recursive_delete_model(agent: AgentClient, model_name: str, nodes=None, log_
                     models_with_pattern = agent.get_models_with_patterns([old_symbol])
 
                     for node1 in nodes:
-                        print(f"models_with_pattern[{node1['name']}] = {models_with_pattern[node1['name']]} with {old_symbol}")
+                        if (log_to_file):
+                            file.write(f"models_with_pattern[{node1['name']}] = {models_with_pattern[node1['name']]} with {old_symbol}")
 
-                        if models_with_pattern[node1["name"]]["model_list"] == []:
+                        if (models_with_pattern[node1["name"]]["model_list"] == []):
                             continue
 
-                        print("node1['name'] not an empty list")
+                        if (log_to_file):
+                            file.write("node1['name'] not an empty list")
 
                         # go through each of them and delete the old model
                         # and add a model with the new symbol
@@ -388,47 +403,187 @@ def recursive_delete_model(agent: AgentClient, model_name: str, nodes=None, log_
                             if model_instance[node1["name"]] == {}:
                                 continue
 
+
                             model_sequence = model_instance[node1["name"]]["sequence"]
                             model_frequency = model_instance[node1["name"]]["frequency"]
                             model_emotives = model_instance[node1["name"]]["emotives"]
 
                             # delete the model
                             response = agent.delete_model(model)
-                            print(f"response to delete_model[{node1['name']}] = {response[node1['name']]} for model = {model}")
+                            if (log_to_file):
+                                file.write(f"response to delete_model[{node1['name']}] = {response[node1['name']]} for model = {model}")
 
                             # don't continue unless it was actually deleted
-                            if response[node1["name"]] != "deleted":
-                                print(f"{response[node1['name']]}")
+                            if(response[node1["name"]] != "deleted"):
+                                if (log_to_file):
+                                    file.write(f"{response[node1['name']]}")
                                 continue
                             else:
-                                print(f"delete old model response = {response}")
+                                if (log_to_file):
+                                    file.write(f"delete old model response = {response}")
 
                             # replace patterns
                             pure_hash = old_symbol
                             pure_hash = pure_hash.replace(".+", "")
                             for i in range(0, len(model_sequence)):
                                 for j in range(0, len(model_sequence[i])):
-                                    if pure_hash in model_sequence[i][j]:
-                                        print(f"replaced {model_sequence[i][j]} with {new_symbol}")
+                                    if (pure_hash in model_sequence[i][j]):
+                                        if (log_to_file):
+                                            file.write(f"replaced {model_sequence[i][j]} with {new_symbol}")
                                         model_sequence[i][j] = model_sequence[i][j].replace(pure_hash, new_symbol)
 
                             # add model with new symbols
-                            response = agent.add_model(sequence=model_sequence,
-                                                       vector=[],
-                                                       frequency=model_frequency,
-                                                       emotives=model_emotives,
-                                                       nodes=[node1['name']])
+                            response = agent.add_model(sequence = model_sequence,
+                                            vector=[],
+                                            frequency = model_frequency,
+                                            emotives = model_emotives,
+                                            nodes=[node1['name']])
 
-                            print(f"response to add_model = {response}")
+                            if (log_to_file):
+                                file.write(f"response to add_model = {response}")
+
+                            changed_models[node1["name"]][model] = response[node1["name"]]["name"]
 
             else:
-                print(f"Added .+\\|{node['id']}\\|{model_name}.+ pattern to primitve_patterns_to_remove")
-                primitve_patterns_to_remove.append(f".+\\|{node['id']}\\|{model_name}.+")
+                if (log_to_file):
+                    file.write(f"Added .+\|{node['id']}\|{model_name}.+ pattern to primitve_patterns_to_remove")
+                primitve_patterns_to_remove.append(f".+\|{node['id']}\|{model_name}.+")
+        
+        except Exception as exception:
+            raise str(exception)
+
+    if (log_to_file):
+        file.close()
+
+    # TODO changed models can be returned, but becomes super messy if you update a model several times
+    return overall_result, changed_models
+
+
+def recursive_update_model(agent: AgentClient, model_name: str,
+                           model: dict = {}, nodes = None, log_to_file: bool = False):
+    """_summary_
+    Args:
+        agent (AgentClient): _description_
+        model_name (str): _description_
+        nodes (_type_, optional): _description_. Defaults to None.
+        log_to_file (bool, optional): _description_. Defaults to False.
+    """
+
+    if (not agent._connected):
+        return "Agent not connected"
+
+    if (log_to_file):
+        import time
+        t = time.localtime()
+        file = open(f"{time.strftime('recursive_model_update_log_%Y_%m_%d_%H_%M_%S', t)}_log.txt", mode='w+')
+
+    # variables
+    changed_models = {}
+    primitve_patterns_update_in_other_nodes = {}
+
+    if nodes is None:
+        nodes = agent.all_nodes
+    else:
+        nodes = [node for node in agent.all_nodes if (node['name'] in nodes)]
+
+    overall_result = {}
+
+    for node in nodes:
+        changed_models[node["name"]] = dict()
+
+    for node in nodes:
+        try:
+            # call delete model on the passed nodes
+            response = agent.update_model(model_name, model, [node["name"]])
+            overall_result[node["name"]] = response[node["name"]]
+
+            if (log_to_file):
+                file.write(f"initial update_model response = {response[node['name']]}\n")
+
+            # if model was not in node, just skip
+            if (response[node['name']].find("failed") != -1):
+                continue
+
+            changed_models[node["name"]] = response[node["name"]]
+
+            # if model_hash did not change, we can simple add it to the list of symbols to update
+            file.write(f"{model=}")
+            model["name"] = response[node['name']]
+            primitve_patterns_update_in_other_nodes[f".+\|{node['id']}\|{model_name}.+"] = model
+
+            if (log_to_file):
+                file.write("Trying to recursive update models\n")
+
+            for node1 in nodes:
+                # current node won't contain a pattern its updated
+                if (node1 == node):
+                    continue
+
+                patterns_to_add = {}
+                for pattern, pattern_data in primitve_patterns_update_in_other_nodes.items():
+                    if (log_to_file):
+                        file.write(f"Looking for pattern {pattern} in {node1['name']}\n")
+                    # need to get all models that contain this model in their sequences
+                    result = agent.get_models_with_patterns([pattern], [node1["name"]])[node1["name"]]["model_list"]
+
+                    if (log_to_file):
+                        file.write(f"Result for {pattern} in {node1['name']} {result=}\n")
+
+                    if (result == []):
+                        continue
+
+                    model_name_to_find = pattern.split("|")[-1][0:-2]
+                    if (log_to_file):
+                        file.write(f"{model_name_to_find=}")
+
+                    # for each model that needs to be updated, we update
+                    for model_name in result:
+                        # get model data
+                        model_data = agent.get_model(model_name)[node1["name"]]
+
+                        # find where the model name appears and replace it
+                        for i in range(len(model_data["sequence"])):
+                            for j in range(len(model_data['sequence'][i])):
+                                if (model_data["sequence"][i][j].find(model_name_to_find) != -1):
+                                    model_data["sequence"][i][j] = \
+                                        model_data["sequence"][i][j].replace(model_name_to_find, pattern_data["name"])
+
+                        # convert emotives in model_data and average it
+                        for key, item in model_data["emotives"].items():
+                            emotive_data = model_data["emotives"][key]
+                            if not (type(emotive_data) is list):
+                                continue
+
+                            model_data["emotives"][key] = sum(emotive_data) / len(emotive_data)
+
+                        # replace emotive values (no way to really average it)
+                        for key, item in pattern_data["emotives"].items():
+                            model_data["emotives"][key] = item
+
+                        # replace metadata
+                        model_data["metadata"] = pattern_data["metadata"]
+
+                        # replace pattern with new data
+                        recur_result = agent.update_model(model, model_data, [node1["name"]])[node1["name"]]
+
+                        # add updated pattern
+                        changed_models[node1["name"]][model_data["name"]] = recur_result
+                        model_data["name"] = recur_result
+                        if (pattern in patterns_to_add):
+                            continue
+                        patterns_to_add[f".+\|{node1['id']}\|{model_name}.+"] = model_data
+
+                for pattern, pattern_data in patterns_to_add.items():
+                    # add updated pattern
+                    if (pattern in primitve_patterns_update_in_other_nodes):
+                        continue
+                    primitve_patterns_update_in_other_nodes[pattern] = pattern_data
 
         except Exception as exception:
             raise str(exception)
 
-    if log_to_file:
+    if (log_to_file):
         file.close()
 
-    return "success"
+    # TODO changed models can be returned, but becomes super messy if you update a model several times
+    return overall_result, changed_models
