@@ -1,5 +1,6 @@
 """Implements a variety of prediction models."""
 from collections import Counter
+from copy import deepcopy
 
 
 def principal_delta(principal, other, potential):
@@ -115,8 +116,8 @@ def make_modeled_emotives(ensemble):
 
     filtered_ensemble = []
     for p in ensemble:
-        new_record = p
-        new_record['emotives'] = average_emotives([p['emotives']])
+        new_record = deepcopy(p)
+        new_record['emotives'] = average_emotives([new_record['emotives']])
         filtered_ensemble.append(new_record)
 
     filtered_ensemble = bucket_predictions(filtered_ensemble)
