@@ -89,7 +89,8 @@ class PerformanceValidationTest():
                  dataset_info: dict = None,
                  test_id=None,
                  test_configuration: dict = {},
-                 socket_channel: str = 'pvt_status'):
+                 socket_channel: str = 'pvt_status',
+                 QUIET: bool = False):
         """Initialize the PVT object with all required parameters for execution
 
         Args:
@@ -151,6 +152,7 @@ class PerformanceValidationTest():
         self.test_configuration                  = test_configuration
         self.labels_counter                      = Counter()
         self.socket_channel                      = socket_channel
+        self.QUIET                               = QUIET
 
         self.agent_genes = {}
         if self.user_id == '':
@@ -169,6 +171,9 @@ class PerformanceValidationTest():
             self.dataset = Data(data_directories=[self.ds_filepath])
         elif dataset_location == 'prepared':
             self.dataset = self.ds_filepath
+        elif dataset_location == 'prepared_obj':
+            self.dataset = self.ds_filepath
+
         else:
             raise Exception(f'unknown value for dataset location: {dataset_location}')
 
@@ -179,18 +184,28 @@ class PerformanceValidationTest():
         self.agent.set_ingress_nodes(nodes=self.ingress_nodes)
         self.agent.set_query_nodes(nodes=self.query_nodes)
 
-        print(f"num_of_tests      = {self.num_of_tests}")
-        print(f"ds_filepath       = {self.ds_filepath}")
-        print(f"pct_of_ds         = {self.pct_of_ds}")
-        print(f"pct_res_4_train   = {self.pct_res_4_train}")
-
         # Setting summarize single to False by default in order to handle multiply nodes topologies
         self.agent.set_summarize_for_single_node(False)
-        print(f"summarize_for_single_node status   = {self.agent.summarize_for_single_node}")
+        
+        if not self.QUIET:
+            print(f"num_of_tests      = {self.num_of_tests}")
+            print(f"ds_filepath       = {self.ds_filepath}")
+            print(f"pct_of_ds         = {self.pct_of_ds}")
+            print(f"pct_res_4_train   = {self.pct_res_4_train}")
+            print(f"summarize_for_single_node status   = {self.agent.summarize_for_single_node}")
 
     def prepare_datasets(self):
         if self.dataset_location == 'prepared':
             self.dataset_location = 'filepath'
+            if not self.QUIET:
+                print(f"Length of Training Set = {len(self.dataset.train_sequences)}\n")
+                print(f"Length of Testing Set  = {len(self.dataset.test_sequences)}\n")
+            return
+
+        elif self.dataset_location == 'prepared_obj':
+            if not self.QUIET:
+                print(f"Length of Training Set = {len(self.dataset.train_sequences)}\n")
+                print(f"Length of Testing Set  = {len(self.dataset.test_sequences)}\n")
             return
 
         self.dataset.prep(
@@ -198,14 +213,17 @@ class PerformanceValidationTest():
             percent_reserved_for_training=self.pct_res_4_train,
             shuffle=self.shuffle
         )
-        print(f"Length of Training Set = {len(self.dataset.train_sequences)}\n")
-        print(f"Length of Testing Set  = {len(self.dataset.test_sequences)}\n")
+        if not self.QUIET:
+            print(f"Length of Training Set = {len(self.dataset.train_sequences)}\n")
+            print(f"Length of Testing Set  = {len(self.dataset.test_sequences)}\n")
+        return
 
     def run_classification_pvt(self):
         for test_num in range(0, self.num_of_tests):
             self.test_num = test_num
-            print(f'Conducting Test # {test_num}')
-            print('\n---------------------\n')
+            if not self.QUIET:
+                print(f'Conducting Test # {test_num}')
+                print('\n---------------------\n')
 
             self.prepare_datasets()
 
@@ -222,27 +240,36 @@ class PerformanceValidationTest():
 
                 for k, labels in self.labels_set.items():
                     self.labels_set[k] = set([label.rsplit('|', maxsplit=1)[-1] for label in labels])
-                print('Getting Classification Metrics...')
+                if not self.QUIET:
+                    print('Getting Classification Metrics...')
                 self.get_classification_metrics()
-                print('Saving results to pvt_results...')
+                if not self.QUIET:
+                    print('Saving results to pvt_results...')
                 self.pvt_results[f'test_num_{test_num}_metrics'] = self.class_metrics_data_structures
                 self.pvt_results[f'test_num_{test_num}_metrics'] = self.update_test_results_w_hive_classification_metrics(self.pvt_results[f'test_num_{test_num}_metrics'])
 
             except Exception as e:
-                print('error during training/testing phase of test, remediating database for failed test, then raising error')
+                if not self.QUIET:
+                    print('error during training/testing phase of test, remediating database for failed test, then raising error')
                 if self.mongo_results:
-                    print('about to remediate database')
+                    if not self.QUIET:
+                        print('about to remediate database')
                     self.mongo_results.deleteResults()
-                    print('remediated database')
+                    if not self.QUIET:
+                        print('remediated database')
 
-                print(f'raising error {str(e)}')
+                if not self.QUIET:
+                    print(f'raising error {str(e)}')
                 raise e
 
             try:
-                print('Plotting Results...')
-                plot_confusion_matrix(test_num=test_num, class_metrics_data_structures=self.class_metrics_data_structures)
+                if self.dataset_location != 'mongodb':
+                    if not self.QUIET:
+                        print('Plotting Results...')
+                    plot_confusion_matrix(test_num=test_num, class_metrics_data_structures=self.class_metrics_data_structures)
             except Exception as e:
-                print(f'error plotting results from classification pvt: {e}')
+                if not self.QUIET:
+                    print(f'error plotting results from classification pvt: {e}')
                 pass
 
             response_dict = {'classification_counter': self.labels_counter,
@@ -261,8 +288,9 @@ class PerformanceValidationTest():
         self.pvt_results = {}
         for test_num in range(0, self.num_of_tests):
 
-            print(f'Conducting Test # {test_num}')
-            print('\n---------------------\n')
+            if not self.QUIET:
+                print(f'Conducting Test # {test_num}')
+                print('\n---------------------\n')
 
             self.prepare_datasets()
             if self.sio:
@@ -274,13 +302,16 @@ class PerformanceValidationTest():
             self.train_agent()
 
             self.test_agent()
-
-            print('Getting Emotives Value Metrics...')
+            
+            if not self.QUIET:
+                print('Getting Emotives Value Metrics...')
             self.get_emotives_value_metrics()
-            print('Saving results to pvt_results...')
+            if not self.QUIET:
+                print('Saving results to pvt_results...')
             self.pvt_results[f'test_num_{test_num}_metrics'] = self.emotives_metrics_data_structures
             self.pvt_results[f'test_num_{test_num}_metrics'] = self.update_test_results_w_hive_emotives_value_metrics(self.pvt_results[f'test_num_{test_num}_metrics'])
-            print('Plotting Results...')
+            if not self.QUIET:
+                print('Plotting Results...')
 
             # don't try to plot emotive values if we're working to save in a mongo database
             # (its probably running without a jupyter GUI)
@@ -301,9 +332,9 @@ class PerformanceValidationTest():
     def run_emotive_polarity_pvt(self):
         self.pvt_results = {}
         for test_num in range(0, self.num_of_tests):
-
-            print(f'Conducting Test # {test_num}')
-            print('\n---------------------\n')
+            if not self.QUIET:
+                print(f'Conducting Test # {test_num}')
+                print('\n---------------------\n')
             self.prepare_datasets()
 
             if self.sio:
@@ -312,15 +343,19 @@ class PerformanceValidationTest():
                                          metrics={}, cur_test_num=self.test_num, total_test_num=self.num_of_tests,
                                          test_id=self.test_id, user_id=self.user_id, test_type=self.test_type).toJSON(), to=self.user_id)
 
-            print("Training Agent...")
+            if not self.QUIET:
+                print("Training Agent...")
             self.train_agent()
 
-            print("Testing Agent...")
+            if not self.QUIET:
+                print("Testing Agent...")
             self.test_agent()
 
-            print('Getting Emotives Polarity Metrics...')
+            if not self.QUIET:
+                print('Getting Emotives Polarity Metrics...')
             self.get_emotives_polarity_metrics()
-            print('Saving results to pvt_results...')
+            if not self.QUIET:
+                print('Saving results to pvt_results...')
             self.pvt_results[f'test_num_{test_num}_metrics'] = self.emotives_metrics_data_structures
             self.pvt_results[f'test_num_{test_num}_metrics'] = self.update_test_results_w_hive_emotives_polarity_metrics(self.pvt_results[f'test_num_{test_num}_metrics'])
 
@@ -353,15 +388,18 @@ class PerformanceValidationTest():
 
             # Validate Test Type
             if self.test_type == 'classification':
-                print("Conducting Classification PVT...\n")
+                if not self.QUIET:
+                    print("Conducting Classification PVT...\n")
                 self.run_classification_pvt()
 
             elif self.test_type == 'emotives_value':
-                print("Conducting Emotives Value PVT...\n")
+                if not self.QUIET:
+                    print("Conducting Emotives Value PVT...\n")
                 self.run_emotive_value_pvt()
 
             elif self.test_type == 'emotives_polarity':
-                print("Conducting Emotives Polarity PVT...\n")
+                if not self.QUIET:
+                    print("Conducting Emotives Polarity PVT...\n")
                 self.run_emotive_polarity_pvt()
 
             else:
@@ -379,10 +417,11 @@ class PerformanceValidationTest():
                     """
                 )
         except Exception as e:
-            print('\n--------------------------------------------------------------\n')
-            print(f'failed to conduct PVT test, test_type={self.test_type}: {str(e)}')
-            print_exc()
-            print('\n--------------------------------------------------------------\n')
+            if not self.QUIET:
+                print('\n--------------------------------------------------------------\n')
+                print(f'failed to conduct PVT test, test_type={self.test_type}: {str(e)}')
+                print_exc()
+                print('\n--------------------------------------------------------------\n')
             raise e
 
     def train_agent(self):
@@ -393,26 +432,32 @@ class PerformanceValidationTest():
         """
         # Initialize
         if self.clear_all_memory_before_training is True:
-            print('Clearing memory of selected ingress nodes...')
+            if not self.QUIET:
+                print('Clearing memory of selected ingress nodes...')
             self.agent.clear_all_memory(nodes=self.ingress_nodes)
 
         if self.test_type == 'classification':
             # Start an Labels Tracker for each node
-            print('Initialize labels set...')
+            if not self.QUIET:
+                print('Initialize labels set...')
             self.labels_set = {}
             for node in self.ingress_nodes:
                 self.labels_set[node] = set()
-            print(self.labels_set)
+            if not self.QUIET:
+                print(self.labels_set)
             self.labels_counter.clear()
-            print('Created labels set...')
+            if not self.QUIET:
+                print('Created labels set...')
         elif self.test_type == 'emotives_value' or self.test_type == 'emotives_polarity':
             # Start an Emotives Tracker for each node
-            print('Initialize emotives set...')
+            if not self.QUIET:
+                print('Initialize emotives set...')
             self.emotives_set = {}
             for node in self.ingress_nodes:
                 self.emotives_set[node] = set()
-            print(self.emotives_set)
-            print('Created emotives set...')
+            if not self.QUIET:
+                print(self.emotives_set)
+                print('Created emotives set...')
         else:
             raise Exception(
                 """
@@ -434,7 +479,8 @@ class PerformanceValidationTest():
             self.agent.stop_predicting(nodes=self.query_nodes)
         else:
             self.agent.start_predicting(nodes=self.query_nodes)
-        print('Preparing to train agent...')
+        if not self.QUIET:
+            print('Preparing to train agent...')
         # for i, file_path in enumerate(log_progress(dataset.train_sequences)):
 
         train_seq_len = len(self.dataset.train_sequences)
@@ -457,25 +503,31 @@ class PerformanceValidationTest():
             if j % 10 == 0:
                 if self.task:
                     if self.task.is_aborted():
-                        print(f'about to abort {self.task.request.id =}, {self.test_id=}')
+                        if not self.QUIET:
+                            print(f'about to abort {self.task.request.id =}, {self.test_id=}')
                         if self.sio:
-                            print('Sending abort message')
+                            if not self.QUIET:
+                                print('Sending abort message')
                             abort_msg = PVTMessage(status='aborted', current_record=j, total_record_count=train_seq_len,
                                                    metrics={}, cur_test_num=self.test_num + 1, total_test_num=self.num_of_tests,
                                                    test_id=self.test_id, user_id=self.user_id, test_type=self.test_type)
                             self.sio.emit(self.socket_channel, abort_msg.toJSON(), to=self.user_id)
                         if self.mongo_results:
-                            print('cleaning up MongoDB')
+                            if not self.QUIET:
+                                print('cleaning up MongoDB')
                             self.mongo_results.deleteResults()
 
                         return
 
             if j % 100 == 0:
-                print(f"train - {j}")
+                if not self.QUIET:
+                    print(f"train - {j}")
             if self.dataset_location == 'filepath':
                 with open(self.dataset.train_sequences[j], "r") as sequence_file:
                     sequence = sequence_file.readlines()
                     sequence = [json.loads(d) for d in sequence]
+            elif self.dataset_location == 'prepared_obj':
+                sequence = self.dataset.train_sequences[j]
             elif self.dataset_location == 'mongodb':
                 sequence = self.dataset.getSequence(self.dataset.train_sequences[j])
             else:
@@ -493,7 +545,8 @@ class PerformanceValidationTest():
                     self.labels_set[node].update(sequence[-1]['strings'])
                 self.labels_counter.update([label.rsplit('|', maxsplit=1)[-1] for label in sequence[-1]['strings']])
             self.agent.learn(nodes=self.ingress_nodes)
-        print('Finished training agent!')
+        if not self.QUIET:
+            print('Finished training agent!')
 
     def test_agent(self):
         """
@@ -513,31 +566,38 @@ class PerformanceValidationTest():
         test_step_info = {}
         test_seq_len = len(self.dataset.test_sequences)
         if test_seq_len == 0:
-            print('length of testing sequences is 0... returning\n')
+            if not self.QUIET:
+                print('length of testing sequences is 0... returning\n')
             return
         for k, _ in enumerate(self.dataset.test_sequences):
 
             if k % 10 == 0:
                 if self.task:
                     if self.task.is_aborted():
-                        print(f'about to abort {self.task.request.id =}, {self.test_id=}')
+                        if not self.QUIET:
+                            print(f'about to abort {self.task.request.id =}, {self.test_id=}')
                         if self.sio:
-                            print('Sending abort message')
+                            if not self.QUIET:
+                                print('Sending abort message')
                             abort_msg = PVTMessage(status='aborted', current_record=k + 1, total_record_count=test_seq_len,
                                                    metrics={}, cur_test_num=self.test_num + 1, total_test_num=self.num_of_tests,
                                                    test_id=self.test_id, user_id=self.user_id, test_type=self.test_type)
                             self.sio.emit(self.socket_channel, abort_msg.toJSON(), to=self.user_id)
                         if self.mongo_results:
-                            print('cleaning up MongoDB')
+                            if not self.QUIET:
+                                print('cleaning up MongoDB')
                             self.mongo_results.deleteResults()
                         return
 
             if k % 100 == 0:
-                print(f"test - {k}")
+                if not self.QUIET:
+                    print(f"test - {k}")
             if self.dataset_location == 'filepath':
                 with open(self.dataset.test_sequences[k], "r") as sequence_file:
                     sequence = sequence_file.readlines()
                     sequence = [json.loads(d) for d in sequence]
+            elif self.dataset_location == 'prepared_obj':
+                sequence = self.dataset.test_sequences[k]
             elif self.dataset_location == 'mongodb':
                 sequence = self.dataset.getSequence(self.dataset.test_sequences[k])
             else:
@@ -645,18 +705,16 @@ class PerformanceValidationTest():
             answers       = []
             for p in range(0, len(self.predictions)):
                 try:
-                    # print(f'{self.predictions[p][node]=}')
                     overall_pred = prediction_ensemble_model_classification(self.predictions[p][node])
-                    # print(f'{overall_pred=}')
                     if overall_pred is None:
                         # if the agent doesn't have enough information to make a prediction it wouldn't give one
                         overall_preds.append('UNKNOWN')
                     else:
                         overall_preds.append(overall_pred)
                 except Exception as e:
-                    print(f'Something is wrong with the prediction {str(e)}')
+                    if not self.QUIET:
+                        print(f'Something is wrong with the prediction {str(e)}')
                 answers.append(self.actuals[p][0])
-                # print(f'appending {self.actuals[p][0]} to answers')
             try:
                 accuracy = round(accuracy_score(answers, overall_preds), 2) * 100
             except ZeroDivisionError:
@@ -738,7 +796,6 @@ class PerformanceValidationTest():
                     test_step_info['unknown_percentage'][k] = 1 - test_step_info['response_percentage'][k]
 
             else:
-                # print(f'{test_step_info["running_accuracy"]=}, {idx=}')
                 for k in test_step_info['predicted'].keys():
                     if test_step_info['predicted'][k] != 'UNKNOWN':
                         test_step_info['response_counts'][k] += 1
@@ -849,7 +906,6 @@ class PerformanceValidationTest():
             # Not the first observation, so update values
             else:
                 for k in test_step_info['predicted'].keys():
-                    # print(f'{test_step_info["running_accuracy"]=}, {idx=}')
                     for emotive in test_step_info['actual'].keys():
 
                         # catch new emotives, not yet seen on node {k}
@@ -922,7 +978,6 @@ class PerformanceValidationTest():
 
             # Not the first observation, so update values
             else:
-                # print(f'{test_step_info["running_accuracy"]=}, {idx=}')
                 for emotive in test_step_info['actual'].keys():
                     if emotive in test_step_info['predicted'][k].keys():
 
@@ -1118,8 +1173,6 @@ class PerformanceValidationTest():
         for node_name, test_data in pvt_test_result.items():
             if node_name == 'hive':
                 try:
-                    # print(f'{test_data["actuals"]=}, {test_data["predictions"]=}')
-                    # print(f'{copy_hive_preds=}')
                     hive_accuracy = round(accuracy_score(test_data['actuals'], test_data["predictions"]), 2) * 100
                 except ZeroDivisionError:
                     hive_accuracy = 0.0
@@ -1148,8 +1201,6 @@ class PerformanceValidationTest():
                     hive_resp_pc = 0.0
         pvt_test_result['hive']['metrics']['resp_pc'] = hive_resp_pc
 
-        # self.pvt_test_result
-        # pprint.pprint(pvt_test_result)
         return pvt_test_result
 
     def update_test_results_w_hive_emotives_value_metrics(self, pvt_test_result):
@@ -1181,9 +1232,6 @@ class PerformanceValidationTest():
                     if emotive_metric['metrics']['resp_pc'] != 0.0 and emotive in all_nodes_emotives_set:
                         for hive_emotive_metric_dict in hive_emotives_value_metrics_lst:
                             if emotive in hive_emotive_metric_dict.keys():
-                                # print(f'{emotive_metric.keys()}')
-                                # hive_emotive_metric_dict[emotive]['predictions'][node] = emotive_metric['predictions']
-                                # hive_emotive_metric_dict[emotive]['actuals'][node] = emotive_metric['actuals']
                                 hive_emotive_metric_dict[emotive]['metrics']['resp_pc'].append(emotive_metric['metrics']['resp_pc'])
                                 hive_emotive_metric_dict[emotive]['metrics']['rmse'].append(emotive_metric['metrics']['rmse'])
                                 hive_emotive_metric_dict[emotive]['metrics']['smape_prec'].append(emotive_metric['metrics']['smape_prec'])
@@ -1192,7 +1240,6 @@ class PerformanceValidationTest():
         for emotive_metrics in hive_emotives_value_metrics_lst:
             for emotive_name, metric_data in emotive_metrics.items():
                 for metric_name, metric_data_lst in metric_data['metrics'].items():
-                    # print(f'{metric_name=}, {metric_data_lst=}')
                     if len(metric_data_lst) == 0:
                         continue
                     emotive_metrics[emotive_name]['metrics'][metric_name] = sum(metric_data_lst) / len(metric_data_lst)
@@ -1202,9 +1249,7 @@ class PerformanceValidationTest():
         for emo_dict in hive_emotives_value_metrics_lst:
             hive_emotive_overall_dict.update(**emo_dict)
 
-        # print(f'{hive_emotive_overall_dict=}')
         pvt_test_result['hive'] = hive_emotive_overall_dict
-        # pprint.pprint(pvt_test_result)
         return pvt_test_result
 
     def update_test_results_w_hive_emotives_polarity_metrics(self, pvt_test_result):
@@ -1239,7 +1284,6 @@ class PerformanceValidationTest():
         for emotive_metrics in hive_emotives_value_metrics_lst:
             for emotive_name, metric_data in emotive_metrics.items():
                 for metric_name, metric_data_lst in metric_data['metrics'].items():
-                    # print(f'{metric_name=}, {metric_data_lst=}')
                     if len(metric_data_lst) == 0:
                         continue
                     emotive_metrics[emotive_name]['metrics'][metric_name] = sum(metric_data_lst) / len(metric_data_lst)
@@ -1248,16 +1292,16 @@ class PerformanceValidationTest():
         for emo_dict in hive_emotives_value_metrics_lst:
             hive_emotive_overall_dict.update(**emo_dict)
 
-        print(f'{hive_emotive_overall_dict=}')
+        if not self.QUIET:
+            print(f'{hive_emotive_overall_dict=}')
         pvt_test_result['hive'] = hive_emotive_overall_dict
-        # pprint.pprint(pvt_test_result)
         return pvt_test_result
 
     def plot_emotives_value_charts(self, test_num):
 
         for node_name, node_emotive_metrics in self.emotives_metrics_data_structures.items():
-            print(f'-----------------Test#{test_num}-{node_name}-Plots-----------------')
-            # print(f'{node_emotive_metrics = }')
+            if not self.QUIET:
+                print(f'-----------------Test#{test_num}-{node_name}-Plots-----------------')
             for emotive_name, data in sorted(node_emotive_metrics.items()):
                 labels = 'precision', 'miss'
                 if data['metrics']['smape_prec'] is None:
@@ -1277,7 +1321,9 @@ class PerformanceValidationTest():
                 try:
                     plt.savefig(f"{self.results_filepath}/{test_num}_{node_name}_{emotive_name}.png", dpi=300, bbox_inches='tight')
                 except Exception as e:
-                    print(f"Not able to save figure in assigned results directory! Please add an appropriate directory: {str(e)}")
+                    if not self.QUIET:
+                        print(f"Not able to save figure in assigned results directory! Please add an appropriate directory: {str(e)}")
                     pass
                 plt.show()
-                print('---------------------')
+                if not self.QUIET:
+                    print('---------------------')
