@@ -37,6 +37,51 @@ def create_gdf(strings=None,
     .. warning::
         If fields provided are not of the type expected, a warning will be
         raised, but the GDF will still be made with the improper format
+    
+    .. testsetup:: creategdf
+        
+        # here are the expected gdfs
+        gdf1 = {"strings": [],
+                "vectors": [],
+                "emotives": {},
+                "metadata": {}
+                }
+        gdf2 = {"strings": ["hello"],
+                "vectors": [],
+                "emotives": {},
+                "metadata": {}
+                }
+        gdf3 = {"strings": ["hello"],
+                "vectors": [[1, 2, 3, 4]],
+                "emotives": {},
+                "metadata": {}
+                }
+        gdf4 = {"strings": ["hello"],
+                "vectors": [[1, 2, 3, 4]],
+                "emotives": {"utility": 50},
+                "metadata": {}
+                }
+        gdf5 = {"strings": ["hello"],
+                "vectors": [[1, 2, 3, 4]],
+                "emotives": {"utility": 50},
+                "metadata": {"hello": "world"}
+                }
+        from ia.gaius.utils import create_gdf
+        
+    .. doctest:: creategdf
+        :hide:
+        
+        >>> create_gdf() == gdf1
+        True
+        >>> create_gdf(strings=["hello"]) == gdf2
+        True
+        >>> create_gdf(strings=["hello"], vectors=[[1, 2, 3, 4]]) == gdf3
+        True
+        >>> create_gdf(strings=["hello"], vectors=[[1, 2, 3, 4]], emotives={"utility": 50}) == gdf4
+        True
+        >>> create_gdf(strings=["hello"], vectors=[[1, 2, 3, 4]], emotives={"utility": 50}, metadata={"hello": "world"}) == gdf5
+        True
+
     """
     gdf = {
         "vectors": [] if vectors is None else vectors,
@@ -145,6 +190,32 @@ def abstract_names(ensemble: list) -> list:
             ensemble = agent.get_predictions(nodes=['P1'])
             models = abstract_names(ensemble)
 
+    .. testsetup:: abstract_names
+    
+        # example prediction sequences
+        ensemble1 = []
+        ensemble2 = [{"name": "MODEL|1"},
+                     {"name": "MODEL|2"},
+                     {"name": "MODEL|3"},
+                     {"name": "MODEL|4"},
+                     {"name": "MODEL|5"}]
+        ensemble3 = [{"name": "MODEL|0"},
+                     {"name": "MODEL|0"},
+                     {"name": "MODEL|0"},
+                     {"name": "MODEL|0"},
+                     {"name": "MODEL|0"}]
+        from ia.gaius.utils import abstract_names
+
+    .. doctest:: abstract_names
+        :hide:
+        
+        >>> abstract_names(ensemble1) == []
+        True
+        >>> sorted(abstract_names(ensemble2)) == sorted(["MODEL|1", "MODEL|2", "MODEL|3", "MODEL|4", "MODEL|5"])
+        True
+        >>> abstract_names(ensemble3) == ['MODEL|0']
+        True
+
     """
     return list(set([pred['name'] for pred in ensemble]))
 
